#!/bin/sh
############################################################################
#
#  Licensed Materials - Property of IBM
#  Product ID = 5724-C04
#
#  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with
#  IBM Corp.
#
#   File Information: Common/src/tedw_app/amx/fixpack2/misc/amx_after.sh, tmnt_etl, tmnt_5.1.1_FP06, 512ITM03LA
#   Revision: 1.10.3.2
#   Date: 4/6/04 12:15:27
############################################################################

############################################################################
# Command Line Args:
#   1 = the path and name of the TWH Application config file
#   2 = Application working directory or install base directory
#       $TWH_TOPDIR/apps/ava_code_for_app/vapp_version (look at install tree)
#
# Required Config File:
#       twh_app_install.cfg
#
# Installation Logs (Default names):
#       A log will be generated for this script called amx_after.sh.log
#
# Required Env Vars:
#       None.
#
# Return Codes:
#   0 = Success
#   255 = Failure
#
# Interpreter Required:
#       NT/2000 = BASH
#       Unix    = SH (Bourne Shell)
#
# Message Types:
#       (I) - Informational
#       (W) - Warning
#       (F) - Fatal
#
# Install Steps Required Before this Script Executes:
#       6 - TWH Core tools and application components must be
#         successfully installed
#
# What is accomplished:
#  * Copy the amx_create_comp_msmt.sh into $TWH_TOPDIR/tools/bin
#
# Note - Caution needs to be taken for NT paths that have a space.  This
#        is a limitation that requires double quoting around all
#      variables that contain a path.  Testing should be done to ensure
#      this script runs with a space in any of the paths that are used.
#########################################################################
#


###


echo "        ** After Script Log **          "
echo ""
echo "****************************************"
echo "* Application After Script Start point *"
echo "****************************************"
echo ""

# Pulls in the generic shared routines

# Set parms
echo "      Parm 1 = $1"; TWH_CONFIG_FILE="$1"; export TWH_CONFIG_FILE
echo "      Parm 2 = $2"; APP_WORK_DIR="$2"; export APP_WORK_DIR
echo ""

# TWH_TOPDIR needs to always be in the environment
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate Env var TWH_TOPDIR"
  exit 255
fi

# Execute startup driver for afterscript
. "$TWH_TOPDIR"/install/bin/twh_core_startup_after.sh
exec_startup_driver;

#####################################################
######## Add application code below here ONLY #######
#####################################################

###

set_amx_program_vars()
{
  AMX_MISC_DIR="$APP_WORK_DIR/misc"     # path to bash script to be copied
  tmp_sql_file="amx_tmp_sql.sql"        # name of the temporary sql file

  TRUE=1
  FALSE=0

  return 0;
}


###

# The after script of AMX 5.1.2 was not able to correctly register the version
# information in the TDW 1.2 version table, because the database connections
# were truncated after setting the DB parameters.; this prevented the twh_setwepver
# to connect to use the vwkernel services (becuase of the dandling connection handler)
# and then to register the 5.1.1.600 version information. We fix this problem here
# to allow future fixpacks to prerequisite the 5.1.2 level of AMX code.
fix_patch_version()
{
    
    echo "==> Fixing Patch Version 5.1.1.600"
    
    # In a 1.2 environment, the TWH_RPI_HOST variable is still passed to 1.1 scripts, but since 
    # there is no RPI, the value is always set to RPI_NULL_HOST. We can indirectly check the TEDW 
    # version using this environment variable.
    if [ "x$TWH_RPI_HOST" != "xRPI_NULL_HOST" ]; then
        # TEDW 1.1 uses a marker file to track the version of installed WEPs; those markers
        # were successfully created by the 5.1.2 after script, so we don't need to fix anything...
        echo "      (I) You are installing on TEDW 1.1, nothing to do..."
    else
        echo "      (I) You are installing on TDW 1.2, fixing WEP version information"
        twh_setwepver AMX 5.1.1 5.1.1.600
    fi
    if [ $? -ne 0 ]; then
            echo "==> (F) Failed to fix patch version 5.1.1.600"
            echo "        Error is:"
            echo "        $err_txt"
            echo "        Exiting..."
            exit 255
    else
        echo "      (I) Patch 5.1.1.600 successfully fixed"
    fi
}

###

# Prevent from running on wrong host
block_exec_by_host()
{
  echo "==> Validating execution host"

  # for this prototype, only allow this work to be done on the Control Server
  if [ $ADMIN_CHECKED -ne $TRUE ]; then
    echo "      (I) Nothing to be done for this host."
    echo "          Returning control to main executing thread."
    exit 0;
  fi

  echo "      (I) Execution host is valid.  Program will continue."
  return 0;
}


###


# remove previous version of the amx scripts, regardless of the ava_code case
delete_amx_files_in_tools_bin()
{
  echo "==> Remove the AMX files in tools/bin"
  ls "$TWH_TOPDIR"/tools/bin/AMX* | sed 's/\\/\//g' | (
  while read a
  do
    rm -f "$a"
    echo "$a deleted";
  done
  )
}  


###


copy_pre_scripts()
{
  echo "==> Starting to copy the AMX pre scripts"

  err_txt=`cp "$AMX_MISC_DIR/amx_pre_extract.sh" "$TWH_TOPDIR/tools/bin" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_pre_extract.sh script in $TWH_TOPDIR/tools/bin"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi
  
  err_txt=`cp "$AMX_MISC_DIR/amx_extract.sh" "$TWH_TOPDIR/tools/bin" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_extract.sh script in $TWH_TOPDIR/tools/bin"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi

  err_txt=`cp "$AMX_MISC_DIR/amx_create_comp_msmt.sh" "$TWH_TOPDIR/tools/bin" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_create_comp_msmt.sh script in $TWH_TOPDIR/tools/bin"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi

  echo "      (I) Done copying the pre script"
  return 0;
}

copy_amx_rollup()
{
  echo "==> Starting to copy the AMX rollup script"

  err_txt=`cp "$AMX_MISC_DIR/amx_rollup.sh" "$TWH_TOPDIR/tools/bin" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_rollup.sh script in $TWH_TOPDIR/tools/bin"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi

  echo "      (I) Done copying the AMX rollup script"
  return 0;
}


###


copy_amx_uninstall_scripts()
{
  echo "==> Starting to copy the AMX uninstall scripts"

  # remove previous version of the amx scripts, regardless of the ava_code case
  err_txt=`rm -f "$TWH_TOPDIR"/install/bin/amx_* `
  err_txt=`rm -f "$TWH_TOPDIR"/install/bin/AMX_* `

  err_txt=`cp "$AMX_MISC_DIR/amx_app_deinstall.sh" "$TWH_TOPDIR/install/bin" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_app_deinstall.sh script in $TWH_TOPDIR/install/bin"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi

  # remove previous version of the amx scripts, regardless of the ava_code case
  err_txt=`rm -f "$TWH_TOPDIR"/install/sql/amx_* `
  err_txt=`rm -f "$TWH_TOPDIR"/install/sql/AMX_* `

  err_txt=`cp "$AMX_MISC_DIR/amx_delete_tables_list.sql" "$TWH_TOPDIR/install/sql" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_delete_tables_list.sql script in $TWH_TOPDIR/install/sql"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi

  err_txt=`cp "$AMX_MISC_DIR/amx_delete_views_list.sql" "$TWH_TOPDIR/install/sql" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_delete_views_list.sql script in $TWH_TOPDIR/install/sql"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi

  echo "      (I) Done copying the AMX uninstall scripts"
  return 0;
}

remove_amx_uninstall_scripts()
{
  echo "==> Starting to remove the AMX uninstall scripts"
  err_txt=`rm -f "$TWH_TOPDIR"/install/bin/amx_* `
  err_txt=`rm -f "$TWH_TOPDIR"/install/bin/AMX_* `  
  err_txt=`rm -f "$TWH_TOPDIR"/install/sql/amx_* `
  err_txt=`rm -f "$TWH_TOPDIR"/install/sql/AMX_* `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to remove AMX uninstall scripts scripts"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi  
  echo "      (I) Done removing the AMX uninstall scripts"
  return 0;
}


###


set_db2_params()
{
  echo "==> Starting to set DB2 parameters"

  # is the CDW and mart local?
  if [ "$CDW_CHECKED" = "$TRUE" ]; then
    echo "    Local CDW..."

    err_txt=`$DB2EXEC update db cfg for "$TWH_CDW_DB" using APP_CTL_HEAP_SZ 512 2>&1 `
    if [ $? -ne 0 ]; then
      echo "==> (F) Failed to set APP_CTL_HEAP_SZ for $TWH_CDW_DB database"
      echo "        Error is:"
      echo "        $err_txt"
      echo "        Exiting..."
      exit 255
    fi

    tmp_sql_drop_tssys_bp="drop tablespace tssystemp8k;
    drop bufferpool buf8k;"
    tmp_sql_bp="create bufferpool buf8k size 500 pagesize 8 K;"

    # Create a 8K page size bufferpool
    # Drop the sys temp tablespace then the 8K page size bufferpool
    echo "      (I) Drop sys temp tablespace and bufferpool"
    echo "$tmp_sql_drop_tssys_bp" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
    gr_exec_sql_script "$TWH_CDW_DB" $FALSE;
    rm -f "$TMPDIR/$tmp_sql_file"

    # Create it
    echo "      (I) Create bufferpool"
    echo "$tmp_sql_bp" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
    gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
    rm -f "$TMPDIR/$tmp_sql_file"

    # Bounce DB2
    echo "==> Stopping DB2"
    err_txt=`$DB2CMD db2stop force 2>&1 `
    if [ $? -ne 0 ]; then
      blah=`echo $err_txt | grep -i $sql_db_not_started `
      if [ $? -eq 0 ]; then
        :;                  # If db not started that is OK
      else
        echo "==> (F) Failed to stop DB2."
        echo "        Error is:"
        echo "        $err_txt"
        echo "        Exiting..."
        exit 255
      fi
    fi

    # Start DB2
    echo "==> Starting DB2"
    err_txt=`$DB2CMD db2start 2>&1 `

    # Note: ignore errors from db2start
    # Test a connection
    echo "      (I) Testing the DB connection"
    err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
      echo "          Error is:"
      echo "          $err_txt"
      echo "          Exiting..."
      exit 255
    fi

    # Have to do this again in case db2bp thread was forced
    err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
      echo "          Error is:"
      echo "          $err_txt"
      echo "          Exiting..."
      exit 255
    fi

    # create system temp tablespace that uses 8K page size bufferpool
    tmp_sql_tssys="create system temporary tablespace tssystemp8k pagesize 8 K managed by system using ('tssystemp8k') bufferpool buf8k;"

    # Create it
    echo "      (I) Create system temp tablespace"
    echo "$tmp_sql_tssys" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
    gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
    rm -f "$TMPDIR/$tmp_sql_file"

  # We have a remote CDW
  else
    echo "    Remote CDW..."

    # Prep vars
    sql_drop_users="disconnect all;
detach;"

    sql_set_inst_parm="update db cfg for "$TWH_CDW_DB" using APP_CTL_HEAP_SZ 512;
disconnect all;
detach;"

    sql_drop_db_parms="drop tablespace tssystemp8k;
drop bufferpool buf8k;"

    # once created, the bufferpool is activated only for the next connections; before
    # using it, we must diconnect/connect...
    sql_set_db_parms1="create bufferpool buf8k size 500 pagesize 8 K;
disconnect all;"
    sql_set_db_parms2="create system temporary tablespace tssystemp8k pagesize 8 K managed by system using ('tssystemp8k') bufferpool buf8k;"

    # Build and run script to drop users
    echo "      (I) Dropping applications users from CDW"
    echo "$sql_drop_users" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_ATTACH;
    gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
    rm -f "$TMPDIR/$tmp_sql_file"

    # drop users is asynchronous - pause
    sleep 10

    # Build and run script set instance parmeters
    echo "      (I) Setting instance parameters"
    echo "$sql_set_inst_parm" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_ATTACH;
    gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
    rm -f "$TMPDIR/$tmp_sql_file"

    # Build and run script to drop database parameters
    echo "      (I) Dropping DB parameters"
    echo "$sql_drop_db_parms" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
    gr_exec_sql_script "$TWH_CDW_DB" $FALSE;
    rm -f "$TMPDIR/$tmp_sql_file"


    # Build and run script to set db parms1
    echo "      (I) Setting DB parms (bufferpool)"
    echo "$sql_set_db_parms1" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
    gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
    rm -f "$TMPDIR/$tmp_sql_file"

    # Build and run script to set db2 parms2
    echo "      (I) Setting DB parms (system temporary tablespace)"
    echo "$sql_set_db_parms2" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
    gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
    rm -f "$TMPDIR/$tmp_sql_file"

    # Test a connection
    echo "      (I) Testing the DB connection"
    err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
      echo "          Error is:"
      echo "          $err_txt"
      echo "          Exiting..."
      exit 255
    fi

    # Have to do this again in case db2bp thread was forced
    err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
      echo "          Error is:"
      echo "          $err_txt"
      echo "          Exiting..."
      exit 255
    fi

    # Build and run script to drop users (Again)
    echo "      (I) Dropping applications users from CDW - again"
    echo "$sql_drop_users" > "$TMPDIR/$tmp_sql_file"
    gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_ATTACH;
    gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
    rm -f "$TMPDIR/$tmp_sql_file"
  fi

  echo "      (I) Done setting DB parameters"
  return 0;
}

copy_prune_scripts()
{
  echo "==> Starting to copy the AMX prune script"

  err_txt=`cp "$AMX_MISC_DIR/amx_rim_prune.sh" "$TWH_TOPDIR/tools/bin" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_rim_prune.sh script in $TWH_TOPDIR/tools/bin"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi
  echo "      (I) Done copying the prune script"
  return 0;
}

copy_extract_scripts()
{
  echo "==> Starting to copy the AMX extract script"

  err_txt=`cp "$AMX_MISC_DIR/amx_rim_extract.sh" "$TWH_TOPDIR/tools/bin" `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to copy amx_rim_extract.sh script in $TWH_TOPDIR/tools/bin"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi
  echo "      (I) Done copying the extract script"
  return 0;
}

############
### Main ###
############

echo "==> Processing main WPack routines"
echo ""

  set_amx_program_vars;
  block_exec_by_host;
  delete_amx_files_in_tools_bin;
  copy_pre_scripts;
  copy_amx_rollup;
  # we prereq TEDW fixpack3 so we don't need any more AMX specific uninstall scripts
  # copy_amx_uninstall_scripts;
  # but we also have to remove those scripts, if already installed by a previous fixpack
  #remove_amx_uninstall_scripts;
  
# we do not recreate the bufferpool/tablespace in the patch because
# it should already be there from the fresh install...
# set_db2_params;
  
  copy_prune_scripts;
  copy_extract_scripts;

  #fix_patch_version;
  
echo ""
echo "      (I) Done processing main WPack routines"

#####################################
## Stop adding application code #####
#####################################
exec_core_after;        # DO NOT MOVE OR REMOVE!


echo ""
echo "==> Completed executing application after script"
exit 0
