#!/bin/sh
############################################################################
#
#  Licensed Materials - Property of IBM
#  Product ID = 5724-C04
#
#  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with
#  IBM Corp.
#
#   File Information: Common/src/tedw_app/amx/fixpack2/misc/amx_before.sh, tmnt_etl, tmnt_5.1.1_FP06, 512ITM06LA
#   Revision: 1.15
#   Date: 2/5/04 21:33:20
############################################################################

############################################################################
# Command Line Args:
#   1 = the path and name of the TWH Application config file
#   2 = Application working directory or install base directory
#       $TWH_TOPDIR/apps/ava_code_for_app/vapp_version (look at install tree)
#
# Required Config File:
#       twh_app_install.cfg
#
# Installation Logs (Default names):
#       A log will be generated for this script called amx_after.sh.log
#
# Required Env Vars:
#       None.
#
# Return Codes:
#       0 = Success
#       255 = Failure
#
# Interpreter Required:
#       NT/2000 = BASH
#       Unix    = SH (Bourne Shell)
#
# Message Types:
#       (I) - Informational
#       (W) - Warning
#       (F) - Fatal
#
# Install Steps Required Before this Script Executes:
#        - TWH Core tools and application components must be
#          successfully installed
#
# What is accomplished:
#  * Remove AMX_1_Process_dwc_data.inp and AMX_1_Process_dwc_data.tag
#    because the patch installer may fail to do this on case sensitive
#    environments.
#
# Note - Caution needs to be taken for NT paths that have a space.  This
#        is a limitation that requires double quoting around all
#        variables that contain a path.  Testing should be done to ensure
#        this script runs with a space in any of the paths that are used.
#########################################################################

echo "        ** Before Script Log **          "
echo ""
echo "****************************************"
echo "* Application Before Script Start point *"
echo "****************************************"
echo ""

# Pulls in the generic shared routines

# Set parms
echo "      Parm 1 = $1"; TWH_CONFIG_FILE="$1"; export TWH_CONFIG_FILE
echo "      Parm 2 = $2"; APP_WORK_DIR="$2"; export APP_WORK_DIR
echo ""

# TWH_TOPDIR needs to always be in the environment
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate Env var TWH_TOPDIR"
  exit 255
fi

# Execute startup driver for beforescript
. "$TWH_TOPDIR"/install/bin/twh_core_startup_after.sh
exec_startup_driver;

#####################################################
######## Add application code below here ONLY #######
#####################################################

#
# Define local variables used by the before script
#
set_amx_program_vars()
{
  AMX_MISC_DIR="$APP_WORK_DIR/misc"
  tmp_sql_file="amx_tmp_sql.sql"
  return 0;
}

# In order to successfully install the AMX fixpack5 the fix for TEDW
# APAR 46244 must be already present in the TEDW system; this fix was
# initially provided by the interim fix 8 (11LA patch), but since this
# LA patch may be no longher available at the time of this installation,
# we also check for any subsequent intermim fix that addresses the same
# problem.
# This further check is performed looking at the content of a well known
# data file, located into the system temporary directory and containing
# the marker of the appropriate interim fix, as comunicated on request by
# TEDW Customer Support.
check_v112_efix8()
{
  # if no marker exists then it is tdw 12 so do nothing
  if [ ! -f "$TWH_TOPDIR"/.twh_core_v1.1.2.marker -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.3.marker  -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.4.marker  -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.5.marker  -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.6.marker  -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.7.marker  -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.8.marker  -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.9.marker  -a ! -f "$TWH_TOPDIR"/.twh_core_v1.1.10.marker  ]; then
    # do nothing
      echo "TDW 1.2 is installed"
      echo "Program will continue"
  else

  # is the TEDW fixpack3 installed ?
  if [ ! -f "$TWH_TOPDIR"/.twh_core_v1.1.3.marker ]; then
    echo "==> Only TEDW FixPack 2 detected: checking if required TEDW interim fix is installed"
    if [ -f "$TWH_TOPDIR"/.installed/.twh_core_v1.1.2@efix8.marker ]; then
        echo "      (I) Required TEDW interim fix exists"
    else
      # the required LA patch doesn't seem to be installed, check if it has beed superceded
      # by a subsequent LA patch, looking at the content of a data file located into the
      # system temporary directory: Customer Support is supposed to give instruction on how
      # to create and use this datafile.
      echo "==> The required TEDW interim fix doen't seem to be installed, looking for a superceeding one"
      if [ -f "$TMPDIR"/amx_datafile.dat ]; then
        blah=` cat "$TMPDIR"/amx_datafile.dat`
        if [ ! -f "$TWH_TOPDIR/.installed/$blah" ]; then
          echo "      (F) Cannot validate the interim fix declared into the datafile with with a "
          echo "          corresponding interim fix actually installed in TEDW."
          echo "          Contact again TEDW Customer support for further instruction on how to proceed."
          exit 255
        else
          echo "      (I) Required TEDW interim fix exists"
        fi
      else
        echo "      (F) Cannot find any superceeding interim fix for APAR 46244"
        echo "          Contact Customer Support for further info on how to proceed."
        exit 255
      fi
    fi
    echo "      (I) TEDW interim fix checked. Program will continue."
  else
    echo "==> TEDW FixPack 3 or greater detected."
    echo "      (I) No need to check for E-Fix. Program will continue."
  fi

  fi

  return 0;
}

# Remove invalid rpi data.
# The WebLogic WEP erroneously references the AMX WEP Warehouse
# Sources/Targets so if it is installed before the installation
# of the AMX FP05, the TEDW installer fails; we need to fix this
# inconsistency...
remove_invalid_iwh_data()
{

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CTRL_DB" user "$COPT_CTRL_DB2USER" using "$COPT_CTRL_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CTRL_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  echo "==> Removing the invalid AMX entries in IWH tables"

  echo "connect to $TWH_CTRL_DB user $COPT_CTRL_DB2USER using $COPT_CTRL_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMX_MISC_DIR/del_amx_iwh_relations.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
    echo "      (F) Unable to remove the invalid AMX entries in IWH tables"
    echo "      Error is : $err_txt"
    echo "      Errors detected but instructed to continue..."
  else
    rm -f "$TMPDIR/$tmp_sql_file"
    echo "      (I) Invalid AMX entries in IWH tables deleted. Program will continue."
  fi


return 0;
}

#
# Remove the old tag files that use an uppercase AVA_CODE
# and for this may be left in place by the patch installer
# import routine, with the effect of a double import.
#
remove_old_tag_files()
{
  echo "==> Removing the old tag files"
  
  if [ -f "$APP_WORK_DIR"/etl/AMX_1_Process_dwc_data.inp ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/AMX_1_Process_dwc_data.inp`
  fi
  if [ -f "$APP_WORK_DIR"/etl/AMX_1_Process_dwc_data.tag ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/AMX_1_Process_dwc_data.tag`
  fi
  if [ -f "$APP_WORK_DIR"/etl/amx_fixpack500_dwc_data.tag ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/amx_fixpack500_dwc_data.tag`
  fi
    
  echo "      (I) Old tag files removed. Program will continue."
  return 0;
}


# Delete old AMX extract entries
amx_extract_del_old_entries()
{

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  echo "==> Deleting old AMX extract entries in $TWH_CDW_DB"

  echo "connect to $TWH_CDW_DB user $COPT_CDW_DB2USER using $COPT_CDW_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMX_MISC_DIR/amx_extract_del_old_entries.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
   echo "      (I) Unable to Delete old AMX extract entries in $TWH_CDW_DB "
   echo "      Error is : $err_txt"
   echo "      Errors detected but instructed to continue..."
  else
    rm -f "$TMPDIR/$tmp_sql_file"
    echo "      (I) old AMX extract entries deleted. Program will continue."
  fi
return 0;
}

 
remove_old_comp_msmt_file()
{
echo "==> Removing the old comp_msmt script"
if [ -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s040_Comp_Msmt.db2 ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s040_Comp_Msmt.db2`
  fi

  echo "      (I) Old comp_msmt script removed. Program will continue."
  return 0;
}  

remove_old_extract_files()
{
  echo "==> Removing the old extract files"
  if [ -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.db2 ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.db2 `
  fi
  if [ -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.oracle ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.oracle`
  fi
  if [ -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.mssql ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.mssql`
  fi
  if [ -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.sybase ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.sybase`
  fi
  if [ -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.informix ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s010_rim_extract.informix`
  fi
  if [ -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s040_Comp_Msmt.db2 ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/AMX_c05_s040_Comp_Msmt.db2`
  fi  
  for i in `ls "$APP_WORK_DIR"/misc/tools/???_AMX_reset_extract_window.db2`
  do
	rm -f $i
	echo "$i deleted"
  done

  echo "      (I) Old extract files removed. Program will continue."
  return 0;
}

remove_amx_prune_scripts_in_etl_sql()
{
  echo "==> Remove the AMX files in tools/bin"
  ls "$APP_WORK_DIR"/etl/sql/*_c10_s010_?im_?rune.* | sed 's/\\/\//g' | (
  while read a
  do
    rm -f "$a"
    echo "$a deleted";
  done
  )
} 

update_primary_key()
{

  # Test a connection
  echo "      (I) Testing the DB connection"
  echo "connect to $TWH_CDW_DB user $COPT_CDW_DB2USER using $COPT_CDW_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  echo "==> Changing the primary key of AMX.RESOURCE_TRANSL"

  echo "connect to $TWH_CDW_DB user $COPT_CDW_DB2USER using $COPT_CDW_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMX_MISC_DIR/update_primary_key.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
    echo "      (F) Unable to update the primary key in the AMX.RESOURCE_TRANSL table"
    echo "      Error is : $err_txt"
    echo "      Errors detected but instructed to continue..."
  else
    rm -f "$TMPDIR/$tmp_sql_file"
    echo "      (I) primary key of AMX.RESOURCE_TRANSL updated. Program will continue."
  fi


return 0
}


############
### Main ###
############

echo "==> Processing main WPack routines"
echo ""

  set_amx_program_vars;
  # Because we pre_req AMX FP6, we do not need to do the following
  #check_v112_efix8;
  #remove_invalid_iwh_data;
  #remove_old_tag_files;
  #amx_extract_del_old_entries;
  remove_old_extract_files;
  remove_amx_prune_scripts_in_etl_sql;
  # Because the AMX_c05_s040_comp_msmt.db2 script needs to be regenerated
  # we force it by deleting of the old script
  remove_old_comp_msmt_file;

  # Change the primarey key of amx.resource_transl table to allow 
  # multiple resources linked to the same common component type
  update_primary_key;

echo ""
echo "      (I) Done processing main WPack routines"
echo ""

#####################################
## Stop adding application code #####
#####################################

echo "==> Completed executing application before script"

exit 0
