#!perl

###################################################################
#
#  Licensed Materials - Property of IBM
#  Product ID = 5724-C04
#
#  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with
#  IBM Corp.
#
#   File Information: Common/src/tedw_app/amx/fixpack2/misc/amx_rim_extract.pl, tmnt_etl, tmnt_5.1.1_FP06, 512ITMFP01_0622
#   Revision: 1.8
#   Date: 6/21/04 16:52:25
#
###################################################################

###################################################################
# Build the ETL1 extraction scripts with a user defined commit rate
# and a safety buffer to avoid the extraction of "insecure" records
# (records that may be rolled back by a concurrent RIM Handler).
#
# Parameters:
#   AVACODE      - the AVA_CODE of the caller application (amx)
#   SCRIPTNAME   - the name of the script to be executed by the execsql engine (amx_c05_s010_rim_extract)
#   SRCDB        - source database ODBC alias (itm_db)
#   SRCUSER      - source database user (db2admin)
#   SRCPWD       - source database password (*******)
#   TRGDB        - target database ODBC alias (twh_cdw)
#   TRGUSER      - target database user (db2admin)
#   TRGPWD       - target database password (*******)
#   COMMITPERIOD - perform a commit every COMMITPERIOD inserts into the STAGE_*DATA tables
#   APPDIR       - the application root directory ($TWH_TOPDIR/apps/amx)
#   VERSION      - the application versione (v511)
#   VWS_LOGGING  - DB2 logging directory (c:/sqllib/logging)
#
# Returns:
#   -
###################################################################

###################################################################
# parameters passed by the Warehouse Control Center
###################################################################
$ava_code      = $ARGV[0];
$script_name   = $ARGV[1];
$source_ds     = $ARGV[2];
$source_user   = $ARGV[3];
$source_pass   = $ARGV[4];
$target_ds     = $ARGV[5];
$target_user   = $ARGV[6];
$target_pass   = $ARGV[7];
# new parameters added to support the modified extraction strategy
$commit_period = $ARGV[8]; # perform a commit every $commit_period inserts

###################################################################
# additional parameters determined in the wrapper scrips
###################################################################
$appdir        = $ARGV[9];
$version       = $ARGV[10];
$logdir        = $ARGV[11];

# The Warehouse Control Center overwites the system temporary directory stored 
# in TMP and TEMP environment variables
#$tmpdir = "$ENV{TMP}" || "$ENV{TEMP}" or die "A system temporary directory is not defined.";
# so we use the same directory used for logging
$tmpdir = $logdir;

$twh_topdir = "$ENV{TWH_TOPDIR}" or die "TWH_TOPDIR environment variable not defined.";

$scriptpath = "$appdir/$version/etl/sql";
$misctpath  = "$appdir/$version/misc";

$tmpsrc = "$tmpdir/tmp_src";
$tmplog = "$tmpdir/tmp_log";

#
# template definitions of the data movement statements
#
$insert_metricsdata_template = "
--#INSERT_INTO_TARGET
INSERT INTO AMX.STAGE_METRICSDATA  
SELECT 
 O.MID,
 O.METRICVALUE,
 O.TIMEKEY_DTTM,
 O.IID,     
 O.EID,
 O.INSERT_SEQ
FROM 
 METRICSDATA O
WHERE
 O.INSERT_SEQ > __FROM_INTSEQ__ AND
 O.INSERT_SEQ <= __TO_INTSEQ__
;
";

$insert_categoriesdata_template = "
--#INSERT_INTO_TARGET
INSERT INTO AMX.STAGE_CATEGORIESDATA1  
SELECT 
 O.CID,
 O.CATEGORYVALUE,
 O.TIMEKEY_DTTM,
 O.IID,     
 O.EID,
 O.INSERT_SEQ
FROM 
 CATEGORIESDATA O
WHERE
 O.INSERT_SEQ > __FROM_INTSEQ__ AND
 O.INSERT_SEQ <= __TO_INTSEQ__  
;
";

$aggregate_categoriesdata_records = "
--#EXECUTE_AT_TARGET
INSERT INTO AMX.STAGE_CATEGORIESDATA  
SELECT 
 O.CID,
 O.CATEGORYVALUE,
 MIN(O.TIMEKEY_DTTM),
 O.IID,     
 O.EID
FROM 
 AMX.STAGE_CATEGORIESDATA1 O
GROUP BY
 O.CID,
 O.CATEGORYVALUE,
 O.IID,
 O.EID
;
";

$create_iid_lookup_table = "

--
-- Create a lookup table to easily identify all the instances 
-- related to known WEPs.
--
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR
DROP TABLE AMX.TMP_KNOWN_IID;
--#EXECUTE_AT_TARGET
CREATE TABLE AMX.TMP_KNOWN_IID (
  IID BIGINT NOT NULL
);

--#EXECUTE_AT_TARGET
INSERT INTO AMX.TMP_KNOWN_IID
SELECT
  I.IID AS IID
FROM
  AMX.TMP_WEP_INSTALLED W,
  AMX.STAGE_RMPROFILES P,
  AMX.STAGE1_RESOURCES R,
  AMX.STAGE1_INSTANCES I
WHERE
  W.CATEGORY = P.CATEGORY AND
  P.ID = R.ID AND
  R.RCID = I.RCID
;

--
-- Create an index on the IID field to support the removal of unknown 
-- records from the staging tables.
--
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR
DROP INDEX AMX.IX_TMP_ID;
--#EXECUTE_AT_TARGET
CREATE INDEX AMX.IX_TMP_ID ON AMX.TMP_KNOWN_IID(IID);
";

#
# throw away all the unknown records from the last extracted staging table 
#
$remove_unknown_recods_template = "

--
-- throw away all the unknown records from the AMX.STAGE___TABLENAME__ table
--
--#EXECUTE_AT_TARGET
DELETE FROM AMX.STAGE___TABLENAME__ S
WHERE NOT EXISTS (SELECT 1 FROM AMX.TMP_KNOWN_IID L WHERE S.IID = L.IID)
;

--
-- create a DESC index on the INSERT_SEQ field to speed up the MAX(INSERT_SEQ) evaluation
--
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR
CREATE INDEX AMX.IX___TABLENAME__ ON AMX.STAGE___TABLENAME__(INSERT_SEQ DESC);
";

#
# update extract control table with the MAX(INSERT_SEQ) evaluated 
# on the staging table after throwing away all the unknown records
#
$update_extract_control_template = "

--
-- update the TWG.EXTRACT_CONTROL table according to the MAX(INSERT_SEQ) of the survived data set
--
--#EXECUTE_AT_TARGET
UPDATE TWG.EXTRACT_CONTROL
SET EXTCTL_TO_INTSEQ = (
 SELECT MAX(INSERT_SEQ) FROM AMX.STAGE___TABLENAME__
)
WHERE
  EXTCTL_SOURCE='__TABLENAME__' AND
  EXTCTL_TARGET='AMX.STAGE___TABLENAME__' AND
  (SELECT COUNT(*) FROM AMX.STAGE___TABLENAME__) > 0
;
";

$update_categoriesdata_extract_control = "

--
-- update the TWG.EXTRACT_CONTROL table according to the MAX(INSERT_SEQ) of the survived data set
--
--#EXECUTE_AT_TARGET
UPDATE TWG.EXTRACT_CONTROL
SET EXTCTL_TO_INTSEQ = (
 SELECT MAX(INSERT_SEQ) FROM AMX.STAGE_CATEGORIESDATA1
)
WHERE
  EXTCTL_SOURCE='CATEGORIESDATA' AND
  EXTCTL_TARGET='AMX.STAGE_CATEGORIESDATA' AND
  (SELECT COUNT(*) FROM AMX.STAGE_CATEGORIESDATA1) > 0
;
";

#
# write extract info to the extract log table; this fires the 
# trigger that updates extract control to close the window
#
$log_extraction_template = "
--#EXECUTE_AT_TARGET
INSERT INTO TWG.EXTRACT_LOG
SELECT
 EC.EXTCTL_SOURCE AS EXTLOG_SOURCE,
 EC.EXTCTL_TARGET AS EXTLOG_TARGET,
 CURRENT TIMESTAMP - CURRENT TIMEZONE AS EXTLOG_DONE_DTTM,
 EC.EXTCTL_FROM_RAWSEQ AS EXTLOG_FROM_RAWSEQ,
 EC.EXTCTL_TO_RAWSEQ AS EXTLOG_TO_RAWSEQ,
 EC.EXTCTL_FROM_INTSEQ AS EXTLOG_FROM_INTSEQ,
 EC.EXTCTL_TO_INTSEQ AS EXTLOG_TO_INTSEQ,
 EC.EXTCTL_FROM_DTTM AS EXTLOG_FROM_DTTM,
 EC.EXTCTL_TO_DTTM AS EXTLOG_TO_DTTM
FROM 
 TWG.EXTRACT_CONTROL EC
WHERE
 EC.EXTCTL_SOURCE = '__TABLENAME__' AND
 EC.EXTCTL_TARGET = 'AMX.STAGE___TABLENAME__'
; 
";

###################################################################
# Main
###################################################################

# generate the extraction script
genExtractScript($script_name);

# execute the generated extraction script
logit("Invoking the generated extract script.");
my $rc = runexecsql("$scriptpath/$script_name","$logdir/$script_name.log");

if($rc) {
    logit("An error occurred when attempting to execute the generated extract script!");
    exit $rc;
}

###################################################################
# Subroutines
###################################################################

sub genExtractScript {
    
    my $outfilename = shift;
    
    logit("Generating the extract script $outfilename.");
    logit("Using a Commit Rate of $commit_period.");
    
    # get the content of the start and end files, i.e. the file with the extract statements
    # for all the ITM tables except the METRICSDATA and CATEGORIESDATA one.
    my $startfile = "$misctpath/amx_rim_extract.start";
    open (START, "<$startfile") or die "cannot open $startfile!";
    my @startout = <START>;
    close START;

    my $endfile = "$misctpath/amx_rim_extract.end";
    open (END, "<$endfile") or die "cannot open $endfile!";
    my @endout = <END>;
    close END;
            
    # generate the extract script into the amx/v511/etl/sql directory, concatenating
    # the start script and the statements to extract data from METRICSDATA and 
    # CATEGORIESDATA tables in a sliced way.
    
    my $outfile="$scriptpath/$outfilename";
    unlink <"$outfile.*">; # remove previously generated files
    open (SFH, ">$outfile.db2") or die "cannot open $outfile.db2!";
    
    logit ("Appending the initial static section.");
    print SFH @startout;
    
    logit ("Generating the SQL statements to create and fill the lookup table for known IIDs.");
    print SFH $create_iid_lookup_table;
    
    #############
    # METRICSDATA
    #############
    
    logit ("Generating the SQL statements to extract METRICSDATA records.");
    
    # get all the control values with a single db call
    %control_values=getControlValues(METRICSDATA);
    $from_intseq = $control_values{'EXTCTL_FROM_INTSEQ'};
    $last_insert_seq = $control_values{'LAST_INSERT_SEQ'};
    
    print SFH "\n--------------------------\n-- AMX table:  Metricsdata\n--------------------------\n";
    $ret1 = genInsertDataStatements($insert_metricsdata_template);
    logit ("Appending the generated SQL statements to extract METRICSDATA records.");
    print SFH $ret1;

    # generate the SQL statements to throw away all the unknown records from the AMX.STAGE_METRICSDATA table
    ($remove_unknown_recods = $remove_unknown_recods_template) =~ s/__TABLENAME__/METRICSDATA/g;
    logit ("Appending the SQL statements to throw away unknown records from the staging table AMX.STAGE_METRICSDATA.");
    print SFH $remove_unknown_recods;
           
    # update extract control table with the MAX(INSERT_SEQ) evaluated 
    # on the staging table after throwing away all the unknown records
    ($update_extract_control = $update_extract_control_template) =~ s/__TABLENAME__/METRICSDATA/g;
    logit ("Appending the SQL statements to update the Extract_Control table for METRICSDATA.");
    print SFH $update_extract_control;
    
    ($log_extraction = $log_extraction_template) =~ s/__TABLENAME__/METRICSDATA/g;
    logit ("Appending the SQL statement to log the extraction from METRICSDATA.");
    print SFH $log_extraction;

    ################
    # CATEGORIESDATA
    ################
    
    logit ("Generating the SQL statements to extract CATEGORIESDATA records.");
    
    # get all the control values with a single db call
    %control_values=getControlValues(CATEGORIESDATA);
    $from_intseq = $control_values{'EXTCTL_FROM_INTSEQ'};
    $last_insert_seq = $control_values{'LAST_INSERT_SEQ'};
    
    print SFH "\n-----------------------------\n-- AMX table:  Categoriesdata\n-----------------------------\n";
    $ret2 = genInsertDataStatements($insert_categoriesdata_template);
    logit ("Appending the generated SQL statements to extract CATEGORIESDATA records.");
    print SFH $ret2;    

    # generate the SQL statements to throw away all the unknown records from the AMX.STAGE_CATEGORIESDATA1 table
    ($remove_unknown_recods = $remove_unknown_recods_template) =~ s/__TABLENAME__/CATEGORIESDATA1/g;
    logit ("Appending the SQL statements to throw away unknown records from the staging table AMX.STAGE_CATEGORIESDATA1.");
    print SFH $remove_unknown_recods;
    
    # aggregate the CATEGORIESDATA records
    logit ("Aggregate the CATEGORIESDATA records in the AMX.STAGE_CATEGORIESDATA table.");
    print SFH $aggregate_categoriesdata_records;
            
    # update extract control table with the MAX(INSERT_SEQ) evaluated 
    # on the staging table after throwing away all the unknown records    
    #($update_extract_control = $update_extract_control_template) =~ s/__TABLENAME__/CATEGORIESDATA/g;
    #logit ("Appending the SQL statements to update the Extract_Control table for CATEGORIESDATA.");
    #print SFH $update_extract_control;
    print SFH $update_categoriesdata_extract_control;
    
    ($log_extraction = $log_extraction_template) =~ s/__TABLENAME__/CATEGORIESDATA/g;
    logit ("Appending the SQL statement to log the extraction from CATEGORIESDATA.");
    print SFH $log_extraction;
    
    logit ("Appending the final static section.");
    print SFH @endout;
    close SFH;  
        
    # the SQL syntax is valid for each supported RDBMS so we can 
    # simply copy the $tmpsrc.db2 script with different extensions
    copyFile("$outfile.db2","$outfile.oracle");
    copyFile("$outfile.db2","$outfile.mssql");
    copyFile("$outfile.db2","$outfile.informix");
    copyFile("$outfile.db2","$outfile.sybase");
    
    logit ("Generation of the $outfilename extract script completed.");
    
}

#
# Moving huge datasets from the ITM_DB to the CDW database using a single
# statement causes a significative growth of the CDW transaction log; 
# in order to reduce or limit this growth we give the customer the possibility
# to specify a commit rate, slicing the single massive insert into multiple
# and smaller inserts.
#
sub genInsertDataStatements {

    my $insert_template = shift;            

    my $insert_stmt = $insert_template;
    my $gen_statements;
    
    # compute the right boudary of the extraction window according to the specified offset
    $to_intseq = $last_insert_seq; 

    logit ("Generating the SQL statements to extract records in the range ($from_intseq, $to_intseq].");      
    
    if($from_intseq==-1) {
        $width = $to_intseq - ($from_intseq + 1);
    } else {
        $width = $to_intseq - $from_intseq;
    }
    
    $slices = int($width / $commit_period);
    $remainder = $width % $commit_period;

    $current_slice = $slices;

    $current_from_intseq = $from_intseq;
    $current_to_intseq = $to_intseq;

    while($current_slice > 0) {
        
        $current_to_intseq = $current_from_intseq + $commit_period;
        
        if($current_from_intseq==-1) {
            $current_to_intseq++;
        }
        
        logit("Gererating extract statement for the range ($current_from_intseq, $current_to_intseq].");
        $insert_stmt =~ s/__FROM_INTSEQ__/$current_from_intseq/;
        $insert_stmt =~ s/__TO_INTSEQ__/$current_to_intseq/;
        
        # append the generated statement
        $gen_statements = "$gen_statements $insert_stmt";
        
        # reset the original template for next iteration
        $insert_stmt = $insert_template;
        
        # move forward the extract window
        $current_from_intseq = $current_to_intseq;
        
        $current_slice--;
    }
    
    # reset the original template for last processing
    $insert_stmt = $insert_template;
    
    if(!($slices>0 && $remainder==0)) {
        if($width > $commit_period) {
            # only if the window to extract is wider then the single slice
            $from_intseq = $current_from_intseq;
            $to_intseq = $from_intseq + $remainder;
        } 
        logit("Gererating extract statement for the range ($from_intseq, $to_intseq].");
        $insert_stmt =~ s/__FROM_INTSEQ__/$from_intseq/;
        $insert_stmt =~ s/__TO_INTSEQ__/$to_intseq/;    
        # append the generated statement    
        $gen_statements = "$gen_statements $insert_stmt";   
    }

    return $gen_statements;
    
}

#
# retrieve the control data for a given data table
#
sub getControlValues {
    
    my $tab = shift;

    logit ("Retrieving control data for table $tab.");
        
    # DB2
    $db2src = "$tmpsrc.db2";
    open (S1, ">$db2src") or die "cannot open $db2src!";
    print S1 "SELECT LAST_INSERT_SEQ FROM CONTROLTABLE WHERE TABLE_NAME = '$tab';\n\n";
    print S1 "--#EXECUTE_AT_TARGET\nSELECT EXTCTL_FROM_INTSEQ FROM TWG.EXTRACT_CONTROL WHERE EXTCTL_SOURCE='$tab' AND EXTCTL_TARGET='AMX.STAGE_$tab';\n\n";
    close S1;

    # the SQL syntax is valid for each supported RDBMS so we can 
    # simply copy the $tmpsrc.db2 script with different extensions
    copyFile("$db2src","$tmpsrc.oracle");
    copyFile("$db2src","$tmpsrc.sybase");
    copyFile("$db2src","$tmpsrc.informix");
    copyFile("$db2src","$tmpsrc.mssql");    
        
    $rc = runexecsql($tmpsrc, $tmplog);
    
    if ( $rc > 0 ) {
        logit("An error occurred while retrieving control data. Return code = $rc.");
        exit 1;
    }

    # remove the source temporary file
    unlink <"$tmpsrc.*">;
    
    my $_last_insert_seq    = getValue(LAST_INSERT_SEQ,    $tmplog);
    my $_extctl_from_intseq = getValue(EXTCTL_FROM_INTSEQ, $tmplog);
    
    # put all the control values into an associative array
    my %_control_values = ('LAST_INSERT_SEQ',$_last_insert_seq,'EXTCTL_FROM_INTSEQ',$_extctl_from_intseq);

    # remove the temporary log file
    unlink <"$tmplog*">;
    
    return (%_control_values);
    
}

#
# extract the value part of a name=value couple from the log file produced by the execsql engine
#
sub getValue {

    $name = shift;
    $file = shift;
  
    my $value;
    
    # open the log file produced by the execsql engine
    open (LOG, "$file") or die "cannot open $file $!";
    
    while ($currline = <LOG>) {
        $currline =~ s/ //g;
        if($currline =~ /^$name$/) {
             # skip the next line
             $currline = <LOG>;
             # get the value
             $value = <LOG>;
             $value =~ s/ //g; # remove blanks
             chomp($value);  # remove the new line character
        }
    }  
  
    # close the log file
    close LOG;
    
    if(!$value) {
        logit("Cannot determine the value of $name.");
        exit 1; 
    }

    logit ("  $name=$value");
    
    return $value;
}

#
# launch the execsql engine
#
sub runexecsql {
	
    my $script=shift;
    my $log=shift;

    my $rc;
    
    if ( -d "c:/" ) {
        my $cmd="\"$twh_topdir/tools/bin/execsql\" \"$script\" \"$log\" \"$source_ds\" \"$source_user\" \"$source_pass\" \"$target_ds\" \"$target_user\" \"$target_pass\"";
        `\"$cmd\"`;
        $rc = $?;
    } else {
        my $cmd="$twh_topdir/tools/bin/execsql $script $log $source_ds $source_user $source_pass $target_ds $target_user $target_pass";
        `$cmd`;
        $rc = $?;
    }
    
    $rc >>=8;
    return $rc;
    
}

#
# logging subroutine
#
sub logit {
    
   my $logmsg=shift;
   
   my $logfile="$logdir/amx_rim_extract.diag";
   open (LOG, ">>$logfile") or die "cannot open $logfile!";
   my $date=localtime;
   print LOG "$date: $logmsg\n";
   close LOG;
   
}

#
# copy files in a platform independent way
#
sub copyFile {
    
    my $sourcefile = shift;
    my $targetfile = shift;
    
    open(SOURCE, "<$sourcefile") or die "cannot open $sourcefile!";
    open(TARGET, ">$targetfile") or die "cannot open $targetfile!";
    
    while (<SOURCE>) {
         print TARGET $_;
    }
    
    close(SOURCE);
    close(TARGET);
    
}
