#!perl5

###################################################################
#
#  Licensed Materials - Property of IBM
#  Product ID = 5724-C04
#
#  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with
#  IBM Corp.
#
#  File Information: Common/src/tedw_app/amx/fixpack2/misc/amx_rim_prune.pl, tmnt_etl, tmnt_5.1.1_FP06, 512ITMFP01_0616
#  Revision: 1.3
#  Date: 5/26/04 14:09:38
#
###################################################################

# This script generates ANSI-99 SQL scripts to purge the data tables of the
# ITM database (METRICSDATA and CATEGORIESDATA) in slices of variable width.
# This is necessary to reduce as much as possible the duration of each prune
# transaction and thus reducing the growth of the database transaction log.
# The implementation strategy of this prune-script generator is to mantain
# the same interface towards the TEDW control center, i.e. the same numer of
# parameters, retrieviend any additional information from the AMX.Prune_Control
# table created by the AMX in the CDW database.
# All the required arameters are retrieved by the amx_rim_prune.start script,
# which is executed by the execsql engine. The generated log file is then parsed
# to buid a data structure (Hash of Hash) with all the required parameters for
# both the data tables.

use vars;

# params from the warehouse control center
our $script_name     = $ARGV [0];
our $source_dsn      = $ARGV [1];
our $source_user     = $ARGV [2];
our $source_password = $ARGV [3];
our $target_dsn      = $ARGV [4];
our $target_user     = $ARGV [5];
our $target_password = $ARGV [6];

our $tmpdir = $ENV{'VWS_LOGGING'};
$tmpdir=~s/\\/\//g;

our $twh_topdir = $ENV{'TWH_TOPDIR'};
$twh_topdir=~s/\\/\//g;

our $logdir = $tmpdir;

our $fulldir = $ENV{'FULLDIR'};

my %HoH = getPruneControlData();
logit("The prune control data is:");
logit(HoHtoString(%HoH));

deleteit('METRICSDATA');
deleteit('CATEGORIESDATA');

logPrune();

#
# delete data from the specified table
#
sub deleteit {
    my $tab = shift;

    $tmpfile = "$tmpdir/$script_name-$tab.db2";
    $tmpscr  = "$tmpdir/$script_name-$tab";
    $logfile = "$tmpdir/$script_name-$tab.log";

    my $min_timekey = $HoH{$tab}{"MIN_TIMEKEY"};
    my $max_timekey = $HoH{$tab}{"MAX_TIMEKEY"};

    # get the prune_period from the prune control table
    $prune_period = $HoH{$tab}{"PRUNE_PERIOD"};

    if($min_timekey eq "-") {
        logit("Nothing to delete for table $tab");
        return
    } else {
        my $num_slices = int(($max_timekey - $min_timekey)/$prune_period);
        if(($num_slices lt "0") || (($min_timekey + $prune_period) ge $max_timekey)) {
            $num_slices="1"
        }
        logit("Generating $num_slices delete statements for table: $tab");
    }

    open (S2, ">$tmpfile") or die "cannot open $tmpfile $!";

    my $right = $min_timekey + $prune_period;
    while ($right < $max_timekey) {
        print S2 "DELETE FROM $tab WHERE TIMEKEY < $right;\n";
        $right += $prune_period;
    }
    print S2 "DELETE FROM $tab WHERE TIMEKEY <= $max_timekey;\n";

    close S2;

    logit("Excecuting delete statements for table $tab");

    copyFile("$tmpfile", "$tmpscr.oracle");
    copyFile("$tmpfile", "$tmpscr.informix");
    copyFile("$tmpfile", "$tmpscr.mssql");
    copyFile("$tmpfile", "$tmpscr.sybase");

    my $rc = runexecsql ($tmpscr,$logfile);

    if ( $rc > 0 ) {
        logit("$tmpscr ended with errors. \nReturn code=$rc.");
        exit 1;
    }

    return;
}

#
# log the pruning activity into the AMX.Prune_Log table
#
sub logPrune {

    my $tmpscr  = "$tmpdir/tmp_log_prune";
    my $logfile = "$logdir/tmp_log_prune.log";
    copyFile("$fulldir/amx_rim_prune.end","$tmpscr.db2");
    copyFile("$fulldir/amx_rim_prune.end","$tmpscr.oracle");
    copyFile("$fulldir/amx_rim_prune.end","$tmpscr.informix");
    copyFile("$fulldir/amx_rim_prune.end","$tmpscr.sybase");
    copyFile("$fulldir/amx_rim_prune.end","$tmpscr.mssql");

    logit("Logging the ITM database prune execution into the AMX.Prune_Log table");

    my $rc=runexecsql($tmpscr,$logfile);

    if ( $rc > 0 ) {
        logit("tmp_log_prune failed when updating prune log table. Return code=$rc.");
        exit 1;
    }

    unlink <$tmpscr.*>;

    return;
}


#
# get the prune control data from the amx.prune_control table
#
sub getPruneControlData() {

    my $tmpscr  = "$tmpdir/tmp_prune_range";
    my $logfile = "$logdir/tmp_prune_range.log";

    copyFile("$fulldir/amx_rim_prune.start","$tmpscr.db2");
    copyFile("$fulldir/amx_rim_prune.start","$tmpscr.oracle");
    copyFile("$fulldir/amx_rim_prune.start","$tmpscr.informix");
    copyFile("$fulldir/amx_rim_prune.start","$tmpscr.sybase");
    copyFile("$fulldir/amx_rim_prune.start","$tmpscr.mssql");

    logit("Evaluating prune interval and prune period");

    my $rc=runexecsql($tmpscr,$logfile);

    if ( $rc > 0 ) {
        logit("tmp_prune_range failed when selecting prune range. Return code=$rc.");
        exit 1;
    }

    my @fields = ('TABLE_NAME','MIN_TIMEKEY','MAX_TIMEKEY','PRUNE_PERIOD');
    my $key = 'TABLE_NAME';
    my %HoH = getValues(\@fields,$key,$logfile);

    unlink <$tmpscr.*>;

    return %HoH;

}

#
# launch the execsql engine
#
sub runexecsql {
	
    my $script=shift;
    my $log=shift;

    my $rc;
    
    if ( -d "c:/" ) {
        my $cmd="\"$twh_topdir/tools/bin/execsql\" \"$script\" \"$log\" \"$source_dsn\" \"$source_user\" \"$source_password\" \"$target_dsn\" \"$target_user\" \"$target_password\"";
        `\"$cmd\"`;
        $rc = $?;
    } else {
        my $cmd="$twh_topdir/tools/bin/execsql $script $log $source_dsn $source_user $source_password $target_dsn $target_user $target_password";
        `$cmd`;
        $rc = $?;
    }
    
    $rc >>=8;
    return $rc;
    
}

#
# logging subroutine
#
sub logit {

   my $logmsg=shift;

   my $logfile="$logdir/amx_rim_prune.diag";
   open (LOG, ">>$logfile") or die "cannot open $logfile!";
   my $date=localtime;
   print LOG "$date: $logmsg\n";
   print "$logmsg\n";
   close LOG;

}

#
# Get the resultset returned by a "select * from table" statement executed by the
# execsql engine; data is returned in a tabular format:
#
#  TABLE_NAME            MIN_TIMEKEY  MAX_TIMEKEY PRUNE_PERIOD
#  --------------------  -----------  ----------- ------------
#  METRICSDATA           1056549637   1059829174  3600
#  CATEGORIESDATA        1056549648   1059829174  3600
#
# the result set is stored into an hash of hash, where each record
# is indexed by the value of the key field.
#
# Params:
#  - fields:  the list of fields
#  - key:     the label of the key field
#  - logfile: the log file produced by the execsql engine
#
# Returns:
#  - an hash table, indexed by the key field, of hash tables holding
#    the values of the remaining fields, indexed by the field labels
#
sub getValues() {

    my $fieldsref = $_[0];
    my @m_fields = @$fieldsref;
    my $m_key = $_[1];
    my $logfile = $_[2];

    for(my $i=0; $i<=$#m_fields; $i++) {
      $fields_pattern.=$m_fields[$i];
    }

    # some RDBMS retruns lower/mixed case field labels...
    $fields_pattern=uc($fields_pattern);

    open (LOG, "$logfile") or die "cannot open $logfile!";

    my @positions;
    my %HoH;

    LINE:while ($currline = <LOG>) {
	    $currline=uc($currline);
        $trimmed_currline = $currline;
        $trimmed_currline =~ s/ //g;
        if($trimmed_currline =~ /^$fields_pattern$/) {
            for(my $i=0; $i<=$#m_fields; $i++) {
                push @positions, index($currline,uc($m_fields[$i]));
            }
            $currline = <LOG>;
            while ($currline = <LOG>) {
	            $currline=uc($currline);
                $trimmed_currline = $currline;
                $trimmed_currline =~ s/ //g;
                if($trimmed_currline=~ /^-+$/) {
                    last LINE;
                }
                for(my $i=0; $i<=$#m_fields; $i++) {
                    if($i < $#m_fields) {
                        $value = substr($currline, $positions[$i], $positions[$i+1]-$positions[$i]);
                    } else {
                        $value = substr($currline, $positions[$i]);
                    }
                    $value =~ s/ //g;
                    chomp($value);
                    if($m_fields[$i] eq $m_key) {
                        $who = $value;
                    } else {
                        $key = $m_fields[$i];
                        $HoH{$who}{$key} = $value;
                    }
                }
            }
        }
    }

    close LOG;

    return %HoH;

}

#
# dump the content of an HoH
#
sub HoHtoString() {

    my %HoH = @_;

    my $strOut;
    $strOut.= "{ ";
    foreach $table ( keys %HoH ) {
       $strOut.="$table { ";
       for $field ( keys %{ $HoH{$table} } ) {
          $strOut.="$field=$HoH{$table}{$field} ";
       }
       $strOut.="} ";
    }
    $strOut.= "} ";

    return $strOut;
}

#
# copy files in a platform independent way
#
sub copyFile {

    my $sourcefile = shift;
    my $targetfile = shift;

    open(SOURCE, "<$sourcefile") or die "cannot open $sourcefile!";
    open(TARGET, ">$targetfile") or die "cannot open $targetfile!";

    while (<SOURCE>) {
         print TARGET $_;
    }

    close(SOURCE);
    close(TARGET);

}
