#!/bin/sh
############################################################################
#
#  Licensed Materials - Property of IBM
#  Product ID = 5724-C04
#
#  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with
#  IBM Corp.
#
#   File Information: Common/src/tedw_app/amy/fixpack5/misc/amy_before.sh, tmnt_etl, tmnt_5.1.1_FP06, 512ITMFP01_0616
#   Revision: 1.3.1.6
#   Date: 4/19/04 11:38:34
############################################################################

############################################################################
# Command Line Args:
#   1 = the path and name of the TWH Application config file
#   2 = Application working directory or install base directory
#       $TWH_TOPDIR/apps/ava_code_for_app/vapp_version (look at install tree)
#
# Required Config File:
#       twh_app_install.cfg
#
# Installation Logs (Default names):
#       A log will be generated for this script called amy_before.sh.log
#
# Required Env Vars:
#       None.
#
# Return Codes:
#       0 = Success
#       255 = Failure
#
# Interpreter Required:
#       NT/2000 = BASH
#       Unix    = SH (Bourne Shell)
#
# Message Types:
#       (I) - Informational
#       (W) - Warning
#       (F) - Fatal
#
# Install Steps Required Before this Script Executes:
#        - TWH Core tools and application components must be
#          successfully installed
#
# What is accomplished:
#  * Remove *_dwc_data.inp files from the etl directory; they are not
#    required by the DB2 iwh2exp2 tool to import the metadata tag file.
#  * Remove the etl2 scripts with the old names.
#
# Note - Caution needs to be taken for NT paths that have a space.  This
#        is a limitation that requires double quoting around all
#        variables that contain a path.  Testing should be done to ensure
#        this script runs with a space in any of the paths that are used.
#########################################################################

echo "        ** Before Script Log **          "
echo ""
echo "****************************************"
echo "* Application Before Script Start point *"
echo "****************************************"
echo ""

# Pulls in the generic shared routines

# Set parms
echo "      Parm 1 = $1"; TWH_CONFIG_FILE="$1"; export TWH_CONFIG_FILE
echo "      Parm 2 = $2"; APP_WORK_DIR="$2"; export APP_WORK_DIR
echo ""

# TWH_TOPDIR needs to always be in the environment
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate Env var TWH_TOPDIR"
  exit 255
fi

# Execute startup driver for beforescript
. "$TWH_TOPDIR"/install/bin/twh_core_startup_after.sh
exec_startup_driver;

#####################################################
######## Add application code below here ONLY #######
#####################################################

#
# Define local variables used by the before script
#
set_amy_program_vars()
{
  AMY_MISC_DIR="$APP_WORK_DIR/misc"
  tmp_sql_file="amy_tmp_sql.sql"        # name of the temporary sql file
  return 0;
}

#
# Remove the imp files from the etl directory; they are not
# required by the DB2 iwh2exp2 tool to import the metadata
# tag file.
#
remove_inp_files()
{
  echo "==> Removing unneeded inp files"
  if [ -f "$APP_WORK_DIR"/etl/*_dwc_data.inp ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/*_dwc_data.inp`
  fi
  echo "      (I) Unneeded inp files removed. Program will continue."
  return 0;
}

# Remove invalid rpi data
remove_invalid_rpi_data()
{

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CTRL_DB" user "$COPT_CTRL_DB2USER" using "$COPT_CTRL_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CTRL_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  echo "==> Removing the invalid AMY data marts and reports"

  echo "connect to $TWH_CTRL_DB user $COPT_CTRL_DB2USER using $COPT_CTRL_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMY_MISC_DIR/rpi_amy_data_delete.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
      echo "      (F) Unable to remove the invalid AMY data marts and reports"
      echo "      Error is : $err_txt"
      echo "      Errors detected but instructed to continue..."
  else
      rm -f "$TMPDIR/$tmp_sql_file"
      echo "      (I) Invalid AMY data marts and reports deleted. Program will continue."
  fi
 
return 0;
}

# Rename FP4 AMY tables
drop_rename_fp4_tables()
{

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_DM_DB" user "$COPT_MART_DB2USER" using "$COPT_MART_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_DM_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  echo "==> Removing the old AMY tables"

  echo "connect to $TWH_DM_DB user $COPT_MART_DB2USER using $COPT_MART_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMY_MISC_DIR/drop_mart_fp4_tables.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
      echo "      (F) Unable to remove the AMY tables"
      echo "          Error is : $err_txt"
      echo "          Errors detected but instructed to continue..."
  else
      rm -f "$TMPDIR/$tmp_sql_file"
      echo "      (I) Old AMY tables removed. Program will continue."
  fi

  echo "==> Renaming the old AMY tables"

  echo "connect to $TWH_DM_DB user $COPT_MART_DB2USER using $COPT_MART_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMY_MISC_DIR/rename_fp4_tables.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
      echo "      (F) Unable to rename the AMY tables"
      echo "          Error is : $err_txt"
      echo "          Errors detected but instructed to continue..."
  else
      rm -f "$TMPDIR/$tmp_sql_file"
      echo "      (I) Old AMY tables renamed. Program will continue."
  fi
  
return 0;
}

# Remove FP4 EXTCTL entries
remove_fp4_extctl_entries_drop_cdw_views()
{

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  echo "==> Remove the AMY FP4 extract control entries"

  echo "connect to $TWH_CDW_DB user $COPT_CDW_DB2USER using $COPT_CDW_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMY_MISC_DIR/remove_fp4_extctl_entries.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
      echo "      (F) Unable to Remove the AMY FP4 extract control entries"
      echo "          Error is : $err_txt"
      echo "          Errors detected but instructed to continue..."
  else
      rm -f "$TMPDIR/$tmp_sql_file"
      echo "      (I) AMY FP4 extract control entries removed. Program will continue."
  fi

  echo "==> Drop the AMY FP4 tables/views"

  echo "connect to $TWH_CDW_DB user $COPT_CDW_DB2USER using $COPT_CDW_DB2PASS;" > "$TMPDIR/$tmp_sql_file";
  cat "$AMY_MISC_DIR/drop_cdw_fp4_tables.sql" >> "$TMPDIR/$tmp_sql_file"
  err_txt=`$DB2EXEC -xtf "\"$TMPDIR\""/$tmp_sql_file`

  ret=$?
  if [ $ret -gt 2 ] ; then
      echo "      (F) Unable to Drop the AMY FP4 tables/views"
      echo "          Error is : $err_txt"
      echo "          Errors detected but instructed to continue..."
  else
      rm -f "$TMPDIR/$tmp_sql_file"
      echo "      (I) AMY FP4 tables/views dropped. Program will continue."
  fi    

return 0;
}

#
# Remove the etl2 scripts with the old names
#
remove_old_etl2_scripts()
{
  echo "==> Removing old etl2 scripts"
  if [ -f "$APP_WORK_DIR"/etl/sql/amy_c05_s010_update.db2 ]
  then
    blah=`rm -f "$APP_WORK_DIR"/etl/sql/amy_c05_s010_update.db2`
  fi
  echo "      (I) Old etl2 scripts removed. Program will continue."
  return 0;
}

#
# Remove old i18n resource bundles
#
remove_old_i18n_bundles()
{
  echo "==> Removing old i18n bundles"
  if [ -f "$APP_WORK_DIR"/i18n/com.ibm.twh.amy.nls@1.1.0.jar ]
  then
    blah=`rm -f "$APP_WORK_DIR"/i18n/com.ibm.twh.amy.nls@1.1.0.jar`
  fi
  if [ -f "$APP_WORK_DIR"/i18n/com.ibm.twh.amy.nls_intl@1.1.0.jar ]
  then
    blah=`rm -f "$APP_WORK_DIR"/i18n/com.ibm.twh.amy.nls_intl@1.1.0.jar`
  fi
  echo "      (I) Old i18n bundles removed. Program will continue."
  return 0;
}

############
### Main ###
############

echo "==> Processing main WPack routines"
echo ""

  set_amy_program_vars;
  # the check for the required TEDW interim fix is already performed by the
  # AMX FP05 pre script, so here we can avoid it because the AMY FP05 declares
  # the AMX FP05 as a prerequisite.
  # check_v112_efix8;
  #remove_inp_files;

  # the extended etl2 scripts were introduced with the 21LA patch;
  # the fixpack5 defines new names for those files, so the scripts
  # with the old names can be removed, as well as the RTM scripts that
  # are no longer needed after the fixpack5
  remove_old_etl2_scripts;

  #remove_old_i18n_bundles;
  
  # In a 1.2 environment, the TWH_RPI_HOST variable is still passed to 1.1 scripts, but since 
  # there is no RPI, the value is always set to RPI_NULL_HOST. We indirectly check the TEDW version
  # using this environment variable and apply the fix on the RPI.* tables only if it is at 1.1 level
  #
  # if [ "x$TWH_RPI_HOST" != "xRPI_NULL_HOST" ]; then
  #   echo "You are installing on TEDW 1.1, an update of the RPI tables is required!"
  #   remove_invalid_rpi_data;
  # else
  #   echo "You are installing on TEDW 1.2, there are no RPI tables to be updated!"
  # fi  
    
  #drop_rename_fp4_tables;
  #remove_fp4_extctl_entries_drop_cdw_views;

echo ""
echo "      (I) Done processing main WPack routines"
echo ""

#####################################
## Stop adding application code #####
#####################################

echo "==> Completed executing application before script"

exit 0
